package Archive::Lha::Decode::LH7;

use strict;
use warnings;
use Archive::Lha::Constants;
use Archive::Lha::Decode::Base ( dicbit => 16 );

1;

__END__

=head1 NAME

Archive::Lha::Decode::LH7

=head1 DESCRIPTION

This is a lh7 decoder. See L<Archive::Lha::Decode> for options and examples.

=head1 AUTHOR

Kenichi Ishigaki, E<lt>ishigaki@cpan.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 by Kenichi Ishigaki.

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut
